$ontext;
email: r.gerlagh@uvt.nl
http:

$offtext;

SETS  TT                Extended time periods                    /1*101/
      T(TT)             simulation time periods
      T0(TT)            calibration period                       /1/
      TFIRST(TT)        First simulation period
      TLAST(TT)         Last simulation period
      I                 atmospheric CO2 boxes            /0*2/
      SCENS             Scenarios                        /'BAU','Markov','CBA','Advanced1','Advanced2'/
      LABELS            labels used for output           /'year','population','output','emis','cum emis','fossil fuel','carbon free','ppmv','temp','SLR','tax','energy price','savings','util pc','cons pc','prod pc','welfare','capital','CO2capital','Tempcapital','damage'/;
;
ALIAS(T2,TT);

SCALARS
* economic parameters
 N               number of years per period                     /10.  /
 alpha           capital output elasticity                       /0.306/
 phi             elasticity of carbon-free supply                /2./
 gammaL          maximum rate of population growth               /0.322/
 beta            short-term extra discounting                    / .8121/
 delta           long-term factor of social time preference      / .9511/
 xiK             capital weight in welfare function
 xiD            temperature weight in welfare function

* economic initial state levels and long-term levels
 YEAR0           'first year in simulation'                    /2010/
 L0              'population in first period [G]'                 /6.9/
 K0              'capital stock in first period [Teuro]'        /115.4/
 Lmax            "maximum population level [G]"                  /11/

* economic initial variables
 Ef0             'first period fossil fuel emissions'            /0.318/
 En0             'first period carbon-free energy'               /0.032/
 Eo0             'first period cement and land use emissions'    /0.07/

* climate parameters
 ppmv0           'preindustrial CO2 concentration'               /275/
 CO2atm0         'preindustrial CO2 atm stock'                   /2.129/
 epsilonD        'adjustment speed of temperature per period'    /0.183/
 epsilonSLR      'adjustment speed of sea level rise per period' /0.05/
 piD             'long-term temperature squared sensititivy'     /4.23/
 piSLR           'long-term sea level rise sensititivy'          /0.5/
 DELTAU          'utility costs of climate damages'              /0.00/
 DELTAY          'output costs of climate damages'               /0.003/

* climate initial states
 D0              'initial temperature squared in K2'             /0.73/
 SLR0            'initial sea level rise in m2'                   /0.04/
;

POSITIVE VARIABLES
* these variables are parameters, but they are allowed to change optimally
 zeta1           shadow price of emissions in utility terms
 sav             savings rate
;

PARAMETERS
* general parameters
  YEAR(TT)
  L(TT)         Level of population and labor
  RESULTS2(LABELS,TT)           bau output for Excell file
  RESULTS3(LABELS,SCENS,TT)           bau output for Excell file
  CHECK(*,TT)             checks
* economic parameters
  Ay(TT)         Level of labour productivity in final goods
  Ae(TT)         Level of energy productivity in final goods
  Af(TT)         level of fossil fuel labour productivity
  An(TT)         level of carbon-free labour productivity
  Eother(TT)    emissions from cement and land use
* atmospheric parameters
  a(I)          'uptake of emitted CO2 in long-term atmosph'    /'0'=0.163, '1'=0.184, '2'=0.449/
  etaS(I)     'CO2 depreciation per period'                   /'0'=0,     '1'=0.074, '2'=0.470/
  xiS(I)       atmospheric CO2 weight in welfare function
  S0(I)         'CO2 initial stocks in 2005'                    /'0'=.304,  '1'=0.250, '2'=0.256/
 ;

$include "productivity.inc";
Ay(TT)=Technology(TT,'Ay');
Ae(TT)=Technology(TT,'Ae');
Af(TT)=Technology(TT,'Af');
An(TT)=Technology(TT,'An');

TFIRST(TT) = YES$(ORD(TT) EQ 2);
TLAST(TT)  = YES$(ORD(TT) EQ CARD(TT)-1);
T(TT) = YES$(ORD(TT) GE 2 AND ORD(TT) LT CARD(TT));

DISPLAY TFIRST, TLAST;

xiK  = alpha/(1-alpha*delta);
xiD = (deltaU+DELTAY/(1-alpha*delta))/(1-delta*(1-epsilonD));
xiS(I) = piD*epsilonD*xiD*delta*(1-etaS(I))/(1-delta*(1-etaS(I)));
zeta1.FX = SUM(I,a(I)*xiS(I)/(delta*(1-etaS(I))));
zeta1.l = zeta1.UP;
sav.FX = beta*delta*xiK/(1+beta*delta*xiK);
sav.L = sav.UP;
YEAR(T0)=YEAR0;
L(T0)=L0;
* still to adjust the above
LOOP(TT,
  YEAR(TT+1) = YEAR(TT)+N;
  L(TT+1)=L(TT)*(1+gammaL*(1-L(TT)/Lmax));
  Eother(TT)=(1.1-ORD(TT)/10)*Eo0;
);
Eother(TT)$(Eother(TT) LT 0)=0;

DISPLAY YEAR,L,Ay,Af,An,xiK, xiD, xiS, zeta1.L, sav.l;

VARIABLES
  OBJ            objective function
  W(TT)          welfare
  U(TT)          utility
  V(TT)          long-term discounted welfare

  Y(TT)          gross output [Teuro]
  K(TT)          capital stock [Teuro]
  E(TT)          total energy use [TtCO2 eq]
  Ef(TT)         fossil fuel use = emissions [TtCO2]
  En(TT)         carbon free energy [TtCO2 eq]
  C(TT)          consumption [Teuro]
  S(TT,I)        atmospheric CO2 stock [TtCO2]
  D(TT)          temperature squared [K2]
  SLR(TT)        sea level rise [m]

  Ly(TT)
  Lf(TT)
  Ln(TT)
  Wage(TT)       wage in utility units
;

POSITIVE VARIABLES K,E,Y,C,slackLF,S,D,E,Ef,En,Ly,Lf,Ln;

EQUATIONS
  WELF           objective definition
  UTIL(TT)        utility definition
  WELF1(TT)       recursive welfare definition
  WELF2(TT)       recursive value function definition
  WELF3(TT)      final period value function definition
  KK1a(TT)        production function
  KK1b(TT)        production function
  KK2(TT)        allocation of output over consumption and savings
  LL(TT)         labour market clearance
  EE(TT)         energy market clearance
  EFE(TT)        fossil fuel energy production
  ENE(TT)        carbon free energy production
  SAVINGS(TT)    savings ratio definition
  FOCLy          FOC for Ly
  FOCLf1          FOC for Lf
  FOCLf2          complementarity for Lf
  SS(TT,I)       CO2 boxes
  DD(TT)         temperature adjustment
  SLREQ(TT)      sea level rise adjustment
 ;

** Equations of the model

WELF..                   OBJ =E= SUM(T$TFIRST(T), W(T));
UTIL(T)..                U(T) =E= LOG(C(T)/L(T))-DELTAU*D(T);
WELF1(TT)$T(TT)..        W(TT) =E= U(TT) + beta*delta*V(TT+1);
WELF2(TT)$T(TT)..        V(TT) =E= U(TT) + delta*V(TT+1);
WELF3(TT+1)$TLAST(TT)..  V(TT+1) =E= xiK*log(K(TT+1))-SUM(I,xiS(I)*S(TT+1,I))-xiD*D(TT+1);

KK1a(TT)$T(TT)..         Y(TT) =E= EXP(-DELTAY*D(TT))*(K(TT)**alpha)*(AY(TT)*LY(TT))**(1-alpha);
KK1b(TT)$T(TT)..         Y(TT) =E= EXP(-DELTAY*D(TT))*(K(TT)**alpha)*(AE(TT)*E(TT))**(1-alpha);
KK2(TT)$T(TT)..          C(TT)+K(TT+1) =L= Y(TT);
LL(TT)$T(TT)..           L(TT) =E= Ly(TT)+Lf(TT)+Ln(TT);
EE(TT)$T(TT)..           E(TT) =E= Ef(TT)+En(TT);
EFE(TT)$T(TT)..          Ef(TT) =E= Af(TT)*Lf(TT);
ENE(TT)$T(TT)..          En(TT) =E= ((PHI+1)/PHI)*(An(TT)*Ln(TT))**(PHI/(PHI+1));

SAVINGS(TT)$T(TT)..      Sav =E= K(TT+1)/Y(TT);
FOCLY(TT)$T(TT)..        Wage(TT)*L(TT) + beta*delta*ZETA1*Af(TT)*Lf(TT) + Wage(TT)*Ln(TT)/phi =E= (1-ALPHA)/(1-Sav);
FOCLF1(TT)$T(TT)..       Wage(TT)/Af(TT)+beta*delta*ZETA1 =E= Wage(TT)*(An(TT)**(-phi/(1+phi)))*(Ln(TT)**(1/(1+phi))) + SLACKLF(TT);
FOCLF2(TT)$T(TT)..       slackLF(TT) * LF(TT) =E= 0;

SS(TT,I)$T(TT)..         S(TT+1,I) =E= a(I)*(Ef(TT)+Eother(TT))+(1-etaS(I))*S(TT,I);
DD(TT)$T(TT)..           D(TT+1) =E= D(TT) + epsilonD*(piD*SUM(I,S(TT+1,I))-D(TT));
SLREQ(TT)$T(TT)..        SLR(TT+1) =E= SLR(TT) + epsilonSLR*(piSLR*D(TT)-SLR(TT));

* model without foc
model IAM /WELF,UTIL,WELF1,WELF2,WELF3,KK1a,KK1b,KK2,LL,EE,EFE,ENE,SS,DD,SLREQ/;
* labour allocation sub-model to determin Ly Lf Ln wage Ef En E
model LASM /WELF,LL,FOCLY,FOCLF1,FOCLF2/
* model with foc
model IAMFOC /all/;

** Solution options

** check labour allocation model
option iterlim = 0;
option reslim = 0;
option solprint = on;
option limrow = 0;
option limcol = 0;

T('1')=YES;
zeta1.fx=0;
obj.UP=1000;
Ln.L(T)=(An(T)**phi)*Af(T)**(-1-phi);
wage.L(T)=(1-ALPHA)*phi/((phi*L(T)+Ln.L(T))*(1-Sav.L));
Ly.L(T)=(Ae(T)/(Ay(T)+Ae(T)*Af(T)))*(Af(T)*(L(T)-Ln.L(T))+((phi+1)/phi)*(An(T)*Ln.l(T))**(phi/(phi+1)));
Lf.L(T)=L(T)-Ly.L(T)-Ln.L(T);
Lf.UP(T)=1.5*Lf.L(T);
Ly.LO(T)=0.75*Ly.L(T);
Ln.LO(T)=0.01*Ln.L(T);
W.UP(T)=0;
SOLVE LASM MAXIMizing OBJ using NLP;
option iterlim = 99999;
option reslim = 99999;
SOLVE LASM MAXIMizing OBJ using NLP;
W.UP(T)=INF;

EF.L(T) = AF(T)*LF.L(T);
EN.L(T) = ((PHI+1)/PHI)*(An(T)*Ln.L(T))**(PHI/(PHI+1));
EN.LO(T) = 0.01*EN.L(T);
E.L(T)=Ef.L(T)+En.L(T);

** calculate the BAU scenario

K.L(T0)=K0;
S.L(T0,I)=S0(I);
D.L(T0)=D0;
K.FX(T0)=K0;
S.FX(T0,I)=S0(I);
D.FX(T0)=D0;
SLR.FX(T0)=SLR0;
LOOP(TT$T(TT),
  Y.L(TT) = EXP(-DELTAY*D.L(TT))*(K.L(TT)**alpha)*MIN(Ae(TT)*E.L(TT),Ay(TT)*Ly.L(TT))**(1-alpha);
  K.L(TT+1) = Sav.L*Y.L(TT);
  C.L(TT) = (1-Sav.L)*Y.L(TT);
  S.L(TT+1,I) = a(I)*(Ef.L(TT)+Eother(TT))+(1-etaS(I))*S.L(TT,I);
  D.L(TT+1) = D.L(TT) + epsilonD*(piD*SUM(I,S.L(TT,I))-D.L(TT));
  SLR.L(TT+1) = SLR.L(TT) + epsilonSLR*(piSLR*D.L(TT)-SLR.L(TT));
  U.L(TT) = LOG(C.L(TT)/L(TT))-DELTAU*D.L(TT);
);
K.LO(T)=0.1*K.L(T);
C.LO(T)=0.1*C.L(T);
E.LO(T)=0.1*E.L(T);
DISPLAY Y.L,K.L,C.L,S.L,D.L,U.L;
OBJ.UP=INF;
option iterlim = 0;
option reslim = 0;
SOLVE IAMFOC MAXIMizing OBJ using NLP;
option iterlim = 99999;
option reslim = 99999;
SOLVE IAMFOC MAXIMizing OBJ using NLP;

** Display of results

RESULTS2('YEAR',TT)=YEAR(TT);
RESULTS2('population',TT)=L(TT);
$BATINCLUDE 'accounting7.inc' 'BAU'

** calculate the Markov Equilibrium scenario

T('1')=NO;
K.FX(TFIRST)=K.L(TFIRST);
S.FX(TFIRST,I)=S.L(TFIRST,I);
D.FX(TFIRST)=D.L(TFIRST);
SLR.FX(TFIRST)=SLR.L(TFIRST);
zeta1.fx = SUM(I,a(I)*xiS(I)/(delta*(1-etaS(I))));
SOLVE IAMFOC MAXIMizing OBJ using NLP;

$BATINCLUDE 'accounting7.inc' 'Markov'
*RESULTS3('tax','Markov',tT)$T(TT)=(1-alpha)*beta*Y.L(TT)/E.L(TT)-(1-alpha)*(1-beta)*Y.L(TT)/(Ly.L(TT)*Af(TT));
CHECK('tax',tT)$T(TT)=Wage.L(TT)/Af(TT) - Wage.L(TT)*(An(TT)**(-phi/(1+phi)))*(Ln.L(TT)**(1/(1+phi)))-RESULTS3('tax','Markov',tt);
DISPLAY CHECK;

** calculate the Pigouvian tax Equilibrium scenario

*xiD = (deltaU+DELTAY/(1-alpha*delta))/(1-delta*(1-epsilonD));
xiD = (DELTAU+DELTAY/(1-sav.L))/(1-(sav.L/alpha)*(1-epsilonD));
xiS(I) = piD*epsilonD*xiD*(sav.L/alpha)*(1-etaS(I))/(1-(sav.L/alpha)*(1-etaS(I)));
zeta1.fx = SUM(I,a(I)*xiS(I)/((sav.L/alpha)*(1-etaS(I))))*(sav.L/alpha)/(beta*delta);
DISPLAY xiD, xiS;
xiD = (deltaU+DELTAY/(1-alpha*delta))/(1-delta*(1-epsilonD));
xiS(I) = piD*epsilonD*xiD*delta*(1-etaS(I))/(1-delta*(1-etaS(I)));
DISPLAY xiD, xiS;

SOLVE IAMFOC MAXIMizing OBJ using NLP;

$BATINCLUDE 'accounting7.inc' 'CBA'

** calculate the optimal tax and savings policy

zeta1.LO = 0.1*zeta1.L;
zeta1.UP = 10*zeta1.L;
W.LO(TT)=W.L(TT)

SOLVE IAMFOC MAXIMizing OBJ using NLP;

$BATINCLUDE 'accounting7.inc' 'Advanced1'

sav.lo = 0.1;
sav.up = 0.9;
SOLVE IAMFOC MAXIMizing OBJ using NLP;
sav.FX = beta*delta*xiK/(1+beta*delta*xiK);

$BATINCLUDE 'accounting7.inc' 'Advanced2'

DISPLAY RESULTS2, RESULTS3;

****************************************************
** output writing
****************************************************

DISPLAY RESULTS2,RESULTS3;

EXECUTE_UNLOAD "IAM.gdx" results2 results3 ;
EXECUTE "GDXXRW.exe IAM.gdx O=simulation_results.xls par=results2 rng=DATA!b2 rdim=1 cdim=1 par=results3 rng=DATA!a6 rdim=2 cdim=1";

$ontext;
$OFFTEXT;
